<?php
/*
Plugin Name: LaTeX2HTML
Plugin URI: http://wordpress.org/extend/plugins/latex2html/
Description: Translate the Latex syntax to HTML+CSS syntax.
Version: 1.0.1
Author: Van Abel
Author URI: http://wamath.sinaapp.com/
License: GPL2
*/
//For debug only

//function for active latex2html
register_activation_hook( __FILE__, 'latex2html_install');
function latex2html_install() {
	/* add wp_option to the database */
	add_option("latex2html_mathjax_custom_cdn", "http://cdn.mathjax.org/mathjax/1.1-latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML-full", '', 'yes');
	add_option("latex2html_mathjax_custom_config", "MathJax.Hub.Config({
  tex2jax: {
    inlineMath: [['$','$'], ['\\(','\\)']]
  }
});
", '', 'yes');
	add_option("latex_preamble", '', '', 'yes');
}
//function for deactive latex2html
register_deactivation_hook( __FILE__, 'latex2html_remove' );
function latex2html_remove() {
	/* delete wp_options of the detabase */
	delete_option('latex2html_mathjax_custom_cdn');
	delete_option('latex2html_mathjax_custom_config');
	delete_option('latex_preamble');
}
if( is_admin() ) {
	/*  add the admin menu */
	add_action('admin_menu', 'latex2html_menu');
}
function latex2html_menu() {
	/* add_options_page( $page_title, $menu_title, $capability, $menu_slug, $function);  */
	add_options_page('LaTeX2HTML Setting Up', 'LaTeX2HTML', 'manage_options','LaTeX2HTML_identifier', 'latex2html_option_page');
}
function latex2html_option_page() {
?>
<div class="wrap">
<h2>LaTeX2HTML Options for MathJaX</h2>
<form method="post" action="options.php">
	<?php wp_nonce_field('update-options'); ?>
	<input type="hidden" name="action" value="update" />
    <input type="hidden" name="page_options" value="latex2html_mathjax_custom_cdn,latex2html_mathjax_custom_config,latex_preamble" />
    <table class="form-table">
        <tr valign="top">
        <th scope="row">Custom MathJaX CDN</th>
        <td><input type="text" name="latex2html_mathjax_custom_cdn" size="50" value="<?php echo esc_url( get_option('latex2html_mathjax_custom_cdn') ); ?>" /></td>
	<td><p>If you leave this blank, the default will be used: 
	<code>http://cdn.mathjax.org/mathjax/1.1-latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML-full</code></p></td>
        </tr>
        <tr valign="top">
        <th scope="row">Custom MathJaX config</th>
        <td><textarea name="latex2html_mathjax_custom_config" cols="50" rows="10"/><?php echo esc_textarea(get_option('latex2html_mathjax_custom_config')); ?></textarea></td>
	<td><p>This text will be placed inside the <code>&lt;script x-mathjax-config&gt;</code> tag</p><p>If you leave this blank, the default will be used: <br/><code>
	MathJax.Hub.Config({<br/>
	&nbsp;&nbsp;tex2jax: {<br/>
    &nbsp;&nbsp;&nbsp;&nbsp;inlineMath: [['$','$'], ['\\(','\\)']]<br/>
    &nbsp;&nbsp;}<br/>
});
</code></p></td>
        </tr>
        <tr valign="top">
        <th scope="row">Custom LaTeX preamble</th>
        <td><textarea name="latex_preamble" cols="50" rows="10"/><?php echo esc_textarea(get_option('latex_preamble')); ?></textarea></td>
	<td><p>A good place to put \newcommand's and \renewcommand's</p><p><strong>Do not us $ signs</strong>, they will be added for you</p><p>E.g.<br/><code>\newcommand{\NN}{\mathbb N}<br/>\newcommand{\abs}[1]{\left|#1\right|}</code></p></td>
        </tr>
    </table>
    <p class="submit">
    <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
    </p>
</form>
<span>
This plugin is free of use, but you can support it by <a href="https://me.alipay.com/vanabel#tab2">leaving a small donation</a> (only <strong>alipay</strong> supported).
</span>
</div>
<?php
}

/*
 * inserts the mathjax configuration and script call
*/
function add_mathjax() {
  $custom_cdn = esc_url( get_option('latex2html_mathjax_custom_cdn') );
  $custom_config = wp_kses( get_option('latex2html_mathjax_custom_config'), array() );
  echo "\n<script type='text/x-mathjax-config'>\n";
  if ( $custom_config ) {
    echo $custom_config;
  } else {
    // note that's supposed to be \\ but since we're in "" we need to redouble
    echo "MathJax.Hub.Config({tex2jax: {inlineMath: [['$','$'], ['\\\\(','\\\\)']]}});\n";
  }
  echo "</script>\n";
  echo "<script type='text/javascript' src='";
  if ( $custom_cdn ) {
    echo $custom_cdn;
  } else {
    echo "http://cdn.mathjax.org/mathjax/latest/MathJax.js?config=TeX-AMS-MML_HTMLorMML";
  }
  echo "'></script>\n";
}

/*
 * inserts the mathjax preamble inside the body and above the content
*/
function add_preamble_adder() {
  $preamble = get_option('latex_preamble');
  if ( $preamble ) {
    $preamble = preg_replace('/\\\\/','\\\\\\\\',$preamble);
?>

<script type='text/javascript'>
  newNode = document.createElement('script');
  newNode.type = "math/tex";
  newNode.innerHTML = '<?php echo esc_js($preamble); ?>';
  document.body.insertBefore(newNode,document.body.firstChild);
</script>
<?php
  }
}

function is_add_mathjax($content){
	if ( is_single() && (preg_match('/\\\\\[(.*?)\\\\\]/ims', $content) || preg_match('/\$(.*?)\$/ims', $content))) {
	add_action('wp_footer', 'add_preamble_adder');
	add_action('wp_footer', 'add_mathjax'); }
	return $content;
}
add_filter('the_content', 'is_add_mathjax');

//add the Donate link
add_filter('plugin_row_meta', 'latex2html_plugin_links', 10, 2);
function latex2html_plugin_links ($links, $file)
{
    if ($file == plugin_basename(__FILE__)) {
        $links[] = '<a href="https://me.alipay.com/vanabel#tab2">' . __('Donate', 'latex2html') . '</a>';
    }
    return $links;
}
//load latex style
add_action('wp_print_styles', 'Latex2HTML_StylesAction');
function Latex2HTML_StylesAction ()
{
    $latex2html_path = WP_PLUGIN_DIR . "latex2html/"; //URL to the plugin directory
    if (! is_admin()) {
        wp_enqueue_style('latex2html', get_bloginfo ( 'wpurl' ) . '/wp-content/plugins/latex2html/latex.css', array(), '0.0.5', 'screen,print');
		wp_enqueue_style('print', get_bloginfo ( 'wpurl' ) . '/wp-content/plugins/latex2html/print.css', array(), '0.0.5', 'print');
    }
}
// translate latex syntax to html syntax
function latex_html($content) {
//since there are while condition, so at most go the $max_time, the program will quit
	$total_time = 1; $max_time = 5000;
//restore the content
	$content_data = $content;
//title, author, address, date
	$content_data = preg_replace('/\\\\(title|author|address|date)\s*\{\s*([^\}]+)\s*\}/ims',
							'<span class="latex_$1">$2</span>', $content_data);
//email in the title
	$content_data = preg_replace('/\\\\email\s*\{\s*([a-zA-Z0-9_\.]+@[a-zA-Z0-9_\.]+)\s*\}/ims', '<span class="latex_email"><a href="mailto:$1">$1</a></span>', $content_data);
//email in the content, not centered
	$content_data = preg_replace('/\\\\mailto\s*\{\s*([a-zA-Z0-9_\.]+@[a-zA-Z0-9_\.]+)\s*\}/ims', '<a href="mailto:$1">$1</a>', $content_data);
//keywords
	$content_data = preg_replace('/\\\\(keywords)\s*\{\s*([^\}]+)\s*\}(<br\s*\/>)?/ims',
							'<span class="latex_$1"><span style="font-variant:small-caps;">Keywords.&#x00A0;</span>$2</span>', $content_data);
// date \today
	$content_data = str_replace('\today', get_the_time('m/d/Y'), $content_data);
//section, subsection, subsubsection
	$sec=0; $subsec = 0; $subsubsec=0;
	$tableofcontent = '<span id="contents"  style="text-align:center; font-size:18px; font-variant:small-caps;display:block;">Contents</span><br />
	<span id="sec:content"><a href="#contents">Contents</a></span><br />';
	$parten = '/((<br\s*\/>\n)*\\\\(section|subsection|subsubsection)\s*\{\s*(.*?)\s*\}(<br\s*\/>\n)*)/ims';
	while($total_time++ < $max_time && preg_match($parten, $content_data, $m)){
		if($m[3] == 'section'){
			$sec++;
			$subsec = 0;
			$subsubsec = 0;
			$content_data = str_replace($m[1],
			'<span class="latex_'.$m[3].'">'.$sec.'.&#x00A0;'.$m[4].'<a id="sec:'.$sec.'"></a></span>
			', $content_data);
			$tableofcontent = $tableofcontent.'<span>&#x00A0;'.$sec.'.&#x00A0;&#x00A0;<a href="#sec:'.$sec.'">'.$m[4].'</a></span><br />';
		}
		elseif($m[3] == 'subsection'){
			$subsec++;
			$subsubsec = 0;
			$content_data = str_replace($m[1],
			'<span class="latex_'.$m[3].'">'.$sec.'.'.$subsec.'.&#x00A0;'.$m[4].'<a id="sec:'.$sec.'.'.$subsec.'"></a></span>
			', $content_data);
			$tableofcontent = $tableofcontent.'<span>&#x00A0;&#x00A0;&#x00A0;'.$sec.'.'.$subsec.'.&#x00A0;&#x00A0;<a href="#sec:'.$sec.'.'.$subsec.'">'.$m[4].'</a></span><br />';
		}
		elseif($m[3] == 'subsubsection'){
			$subsubsec++;
			$content_data = str_replace($m[1],
			'<span class="latex_'.$m[3].'">'.$sec.'.'.$subsec.'.'.$subsubsec.'.&#x00A0;'.$m[4].'<a id="sec:'.$sec.'.'.$subsec.'.'.$subsubsec.'"></a></span>', $content_data);
			$tableofcontent = $tableofcontent.'<span>&#x00A0;&#x00A0;&#x00A0;&#x00A0;&#x00A0;'.$sec.'.'.$subsec.'.'.$subsubsec.'.&#x00A0;&#x00A0;<a href="#sec:'.$sec.'.'.$subsec.'.'.$subsubsec.'">'.$m[4].'</a></span><br />';
		}
	}
//replace of \tableofcontents
	$content_data = str_replace('\tableofcontents', $tableofcontent, $content_data);
//remove \label for equation and theorems but keep {\label{*}}
	$content_data = preg_replace('/\\\\label\{(.*?)\}/ims', '', $content_data);
//remove <br />  for theorem environment
	$content_data = preg_replace('/\\\\begin\s*\{\s*(abstract|thm|lem|cor|defn|prop|rem|prob|examp|answer|proof|excs)\s*\}(\[(.*?)\])?(<br\s*\/>)?/ims', '\begin{$1}$2', $content_data);
	$content_data = preg_replace('/(<br\s*\/>\n?)?\\\\end\{(abstract|thm|lem|cor|defn|prop|rem|prob|examp|answer|proof|excs)\}/ims',
	'\end{$2}', $content_data);
	$content_data = preg_replace('/\\\\end\{(abstract|thm|lem|cor|defn|prop|rem|prob|examp|answer|proof|excs)\}(<br\s*\/>\n?)?/ims',
	'\end{$1}', $content_data);
//process theorem, lemma, corollary, definition, propersition, remark, problem, example, answer, proof's "[]".
	$parten = '/(\\\\begin\s*\{\s*(thm|lem|cor|defn|prop|rem|prob|examp|answer|proof|excs)\s*\}\[(.*?)\])(.*?)(\\\\end\s*\{\s*\2\s*\})/ims';
	$thm_content = '\begin{$2}<span style="font-weight:700; font-style:normal;">($3)&#x00A0;</span>$4\end{$2}';
	$content_data = preg_replace($parten, $thm_content, $content_data);
//abstract, theorem, lemma, corollary, definition, propersition, remark, problem, example, answer, proof
	$thm_num = 1; $rem_num = 1; $prob_num = 1; $prob_answer=  1; $answer_num = 1; $examp_num = 1; $excs_num = 1;
	$thmenv = array(
					'sec'		=>	'Section',
					'fig'		=>	'Figure',
					'thm'		=>	'Theorem',
					'lem'		=>	'Lemma',
					'cor'		=>	'Corollary',
					'defn'		=>	'Definition',
					'prop'		=>	'Proposition',
					'rem'		=>	'Remark',
					'proof'		=>	'Proof',
					'prob'		=>	'Problem',
					'examp'		=>	'Example',
					'answer'	=>	"Answer",
					'excs'		=>	'Excise',
					'abstract'	=>	'Abstract'
				);
	$parten = '/(\\\\begin\s*\{\s*(abstract|thm|lem|cor|defn|prop|rem|prob|examp|answer|proof|excs)\s*\}(.*?)(\\\\end\s*\{\s*\2\s*\}))/ims';
	while($total_time++ < $max_time && preg_match($parten, $content_data, $m)){
		if($m[2] == 'thm'|| $m[2] == 'lem'|| $m[2] == 'cor'|| $m[2] == 'defn'|| $m[2] == 'prop'){
			$content_data = str_replace($m[1],
			'<a id="'.$m[2].':'.$thm_num.'"></a>
			<div class="latex_'.$m[2].'"><span style="font-weight:700;font-style:normal">'.$thmenv[$m[2]].'&#x00A0;'.$thm_num.'.&#x00A0;</span>'.$m[3].'</div>', $content_data);
			$thm_num++;
		}
		elseif($m[2] == 'rem'){
			$content_data = str_replace($m[1],
			'<a id="'.$m[2].':'.$rem_num.'"></a><div class="latex_'.$m[2].'"><span style="font-weight:700; font-style:normal;">'.$thmenv[$m[2]].'&#x00A0;'.$rem_num.'.&#x00A0;</span>'.$m[3].'</div>', $content_data);
			$rem_num++;
		}
		elseif($m[2] == 'proof'){
			$content_data = str_replace($m[1],
			'<span style="font-style:italic;">'.$thmenv[$m[2]].'.&#x00A0;</span>'.$m[3].'<span style="float:right">&#x25A1;</span><br />', $content_data);
		}
		elseif($m[2] == 'prob'){
			$content_data = str_replace($m[1],
			'<a id="'.$m[2].':'.$prob_num.'"></a><div class="latex_'.$m[2].'"><span style="font-weight:700; font-style:normal;">'.$thmenv[$m[2]].'&#x00A0;'.$prob_num.'.&#x00A0;</span>'.$m[3].'</div>', $content_data);
			$prob_num++;
			$prob_answer = $prob_num-1;
			$answer_num = 1;//reset the number of answer by problem.
		}
		elseif($m[2] == 'examp'){
			$content_data = str_replace($m[1],
			'<a id="'.$m[2].':'.$examp_num.'"></a><div class="latex_'.$m[2].'"><span style="font-weight:700; font-style:normal;">'.$thmenv[$m[2]].'&#x00A0;'.$examp_num.'.&#x00A0;</span>'.$m[3].'</div>', $content_data);
			$examp_num++;
		}
		elseif($m[2] == 'answer'){
			$content_data = str_replace($m[1],
			'<a id="'.$m[2].':'.$prob_answer.'.'.$answer_num.'"></a><div class="latex_'.$m[2].'"><span style="font-weight:700; font-style:normal;">'.$thmenv[$m[2]].'&#x00A0;'.$prob_answer.'.'.$answer_num.'.&#x00A0;</span>'.$m[3].'</div>', $content_data);
			$answer_num++;
		}
		elseif($m[2] == 'excs'){
			$content_data = str_replace($m[1],
			'<a id="'.$m[2].':'.$excs_num.'"></a><div class="latex_'.$m[2].'"><span style="font-weight:700; font-style:normal;">'.$thmenv[$m[2]].'&#x00A0;'.$excs_num.'.&#x00A0;</span>'.$m[3].'</div>', $content_data);
			$excs_num++;
		}
		else{
			$content_data = str_replace($m[1],
			'<div class="latex_'.$m[2].'"><span style="font-variant:small-caps;">'.$thmenv[$m[2]].'.&#x00A0;</span>'.$m[3].'</div>', $content_data);
		}
	}

//emphasis
	$content_data = preg_replace('/\\\\(emph|iemph)\s*\{\s*(([^\{\}]*?(\{[^\{\}]*?\})*?)*)\s*\}/ims', '<strong>$2</strong>', $content_data);
//enumerate, itemize
	//remove <br />
	$content_data= preg_replace('/\\\begin\s*\{\s*(itemize|enumerate)\s*\}(<br\s*\/>\n?)?/ims', '\begin{$1}',$content_data);
	$content_data = preg_replace('/\\\\end\{(itemize|enumerate)\}(<br\s*\/>\n?)?/ims', '\end{$1}', $content_data);
	//proceed \item
	$parten = '/(\\\\item\s*(.*?)(<br\s*\/>\n?)?)(\\\\item|\\\\begin\s*\{\s*(itemize|enumerate)\s*\}|\\\\end\s*\{\s*(itemize|enumerate)\s*\})/ims';
	while( $total_time++ < $max_time && preg_match($parten, $content_data, $m)){
		$content_data = str_replace($m[1], '<li>'.$m[2].'</li>', $content_data);
	}
	//proceed \begin \end
	$content_data = preg_replace('/((<\/li>)?\\\\begin\s*\{\s*itemize)\s*\}/ims', '<ul>', $content_data);
	$content_data = preg_replace('/((<\/li>)?\\\\begin\s*\{\s*enumerate)\s*\}/ims', '<ol>', $content_data);
	$content_data = preg_replace('/(\\\\end\s*\{\s*itemize)\s*\}(?=(<li>)|(\\\\end\s*\{\s*itemize\s*\}))/ims', '</ul></li>', $content_data);
	$content_data = preg_replace('/(\\\\end\s*\{\s*enumerate)\s*\}(?=(<li>)|(\\\\end\s*\{\s*enumerate\s*\}))/ims', '</ol></li>', $content_data);
	$content_data = preg_replace('/(\\\\end\s*\{\s*itemize)\s*\}/ims', '</ul>', $content_data);
	$content_data = preg_replace('/(\\\\end\s*\{\s*enumerate)\s*\}/ims', '</ol>', $content_data);
	//$content_data = preg_replace('/<p>\s*(<(ul|ol)>(.*?)<\/\2>)\s*<\/p>/ims', '$1', $content_data);

	//the following code maybe more properly, but I don't know why it is not work for nested environment
	//Eg. the following code
	///* the code */
	//\begin{enumerate}
	//\item first
	//\end{enumerate}
	//\begin{enumerate}
	//\item hah
	//\begin{enumerate}
	//\item first of frist
	//\end{enumerate}
	//\item something
	//\end{enumerate}
	//* end of the code */
	// $parten = '/(\\\\begin\s*\{\s*(enumerate|itemize)\s*\})((.*?)(\\\\begin\s*\{\s*(enumerate|itemize)\s*\}\s*(.*?)\\\\end\s*\{\s*\6\s*\})*)*(\\\\end\s*\{\s*\2\s*\})/ims';
	// while( $total_time++ < $max_time && preg_match($parten, $content_data, $m) ){
		// if($m[2] == 'itemize'){
			// $content_data = str_replace($m[1], '<ul>', $content_data);
			// $content_data = str_replace($m[8], '</ul>',$content_data);
		// }
		// elseif($m[2] == 'enumerate'){
			// $content_data = str_replace($m[1], '<ol>', $content_data);
			// $content_data = str_replace($m[8], '</ol>',$content_data);
		// }
	// }
//quote
 $content_data = preg_replace('/(<br\s*\/>\n?)?\\\\begin\s*\{\s*quote\s*\}(<br\s*\/>\n?)?(.*?)(<br\s*\/>\n?)?\\\\end\s*\{\s*quote\s*\}(<br\s*\/>\n?)?/ims', '<blockquote>$3</blockquote>', $content_data);
//auto-numbering
	$eqn = 1;
 	//search parten for the non-standard form of shoule be numberd equation, i.e.,
	//wrapped with equation, align, multline or gather, and not have a \label inside the {}
	$parten = '/(\\\\begin\s*\{\s*(equation|align|multline|gather)\s*\}(?!\{(\\\\label|\\\\notag))(.*?)\\\\end\s*\{\s*\2\s*\})/ims';
	while( $total_time++ < $max_time && preg_match($parten, $content_data, $m)){
		//reset equation links and equation labels
		$eqn_link = ''; $eqn_label = '';
		//search whether there is a \tag or \notag inside the body
		$tag_parten = '/(\\\\tag\s*\{\s*((\d+)(\.[a-z])*)\s*\}\s*|\\\\notag)/ims';
		if(preg_match_all($tag_parten, $m[4], $tag_data)){
			$i = 0;
			while($total_time++ < $max_time &&  $tag_data[0][$i]){
				//since the $tag_content will be \tag{num} or \notag
				// we need to get the num inside if it is \tag{num}
				if(preg_match($tag_parten, $tag_data[0][$i], $tag_m)){
					//if not \notag
					if($tag_m[1] != '\notag'){
						$eqn = $tag_m[3]+1; //increase the global number of equation
						$eqn_link = $eqn_link.'<a id="eq:'.$tag_m[2].'"></a>'; //added Anchor
						$eqn_label = $eqn_label.'\label{eq:'.$tag_m[2].'}';
					}
					else{
						$eqn_label = $eqn_label.'\notag';
					}
				}
				$i++;
			}
			//added \label and Anchor
			$content_data = str_replace($m[1], $eqn_link.'\begin{'.$m[2].'}{'.$eqn_label.'}'.$m[4].'\end{'.$m[2].'}', $content_data);
		}
		// auto add number for no taged equation
		else{
			$eqn_label = $eqn_label.'\label{eq:'.$eqn.'}';
			$eqn_link = $eqn_link.'<a id="eq:'.$eqn.'"></a>';
			$content_data = str_replace($m[1], $eqn_link.'\begin{'.$m[2].'}{'.$eqn_label.'}'.$m[4].'\tag{'.$eqn.'}\end{'.$m[2].'}', $content_data);
			$eqn++;//increase the global equation number
		}
	}
//ref eqn
	$content_data = preg_replace('/\\\\eqref\s*\{\s*eq:\s*((\d+)(\.[a-z])*)\s*\}/ims', '(<a href="#eq:$1">$1</a>)', $content_data);
//remove dumped 'Theorem', 'Lemma' and so on before the reference
	$dumparry = array('/theorem', '/lemma', '/corollary', '/definition', '/proposition', '/remark', '/proof', '/problem', '/example', '/excise', '/section', '/subsection', '/subsubsection', '/figure');
	foreach ($dumparry as $dumref){
	$parten = $dumref.'(\s*\\\\ref\s*\{\s*(thm|lem|cor|defn|prop|rem|prob|excs|answer|sec|fig):(\d+(\.\d+){0,2})\s*\})/ims';
    $content_data = preg_replace($parten, '$1', $content_data);
	}
	unset($dumref);
// refer the environments
	$parten='/(\\\\ref\s*\{\s*(thm|lem|cor|defn|prop|rem|prob|excs|answer|sec|fig):(\d+(\.\d+){0,2})\s*\})/ims';
	while( $total_time++ < $max_time && $total_time++ < $max_time &&  preg_match($parten, $content_data, $m) ){
		$content_data = str_replace($m[1],
		'<a href="#'.$m[2].':'.$m[3].'">'.$thmenv[$m[2]].' '.$m[3].'</a>', $content_data);
	}
//underline
	$content_data = preg_replace('/\\\\underline\s*\{\s*(([^\{\}]*?(\{[^\{\}]*?\})*?)*)\s*\}/ims', '<span style="text-decoration: underline;">$1</span>', $content_data);
//remove index
	$content_data = preg_replace('/\\\\index\s*\{\s*(([^\{\}]*?(\{[^\{\}]*?\})*?)*)\s*\}/ims', '', $content_data);
//textcolor
	$content_data = preg_replace('/\\\\textcolor\s*\{\s*([a-z]+)\s*\}\{\s*(([^\{\}]*?(\{[^\{\}]*?\})*?)*)\s*\}/ims', '<span style="color:$1">$2</span>', $content_data);
return $content_data;
};

function remove_p($content){
//remove <p> </p> for enviroment
	$content_data=$content;
	$content_data = preg_replace('/(<\/p>)|(<p>)/ims', '<br />', $content_data);
	return $content_data;
};
add_filter('the_content', 'latex_html');
add_filter('the_content', 'remove_p',100);
?>